package com.compo.camera.activities.main;

import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.compo.camera.R;
import com.compo.camera.dialog.ColorPickerDialog;
import com.compo.camera.view.DrawingView;

public class DiscloseMainActivity extends MainActivity
{
	protected static final int COLORS_COLUMNS = 4;
	protected static final int[] COLORS = {
			Color.rgb(255, 51, 51), Color.rgb(255, 151, 0), Color.rgb(255, 226, 12), Color.rgb(53, 184, 46),
			Color.rgb(16, 66, 179), Color.rgb(192, 32, 202), Color.rgb(0, 0, 0), Color.rgb(240, 240, 240),
	};

	protected static final float LOCK_SWIPE_OFFSET = 0.4f;

	protected DrawingView drawing;
	protected Paint drawingPaint;
	protected int drawingColor;

	protected boolean locked;

	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		this.locked = false;

		this.drawingPaint = new Paint();
		this.drawingPaint.setAntiAlias(true);
		this.drawingPaint.setDither(true);
		this.drawingPaint.setColor(this.drawingColor = Color.GREEN);
		this.drawingPaint.setStyle(Paint.Style.STROKE);
		this.drawingPaint.setStrokeJoin(Paint.Join.ROUND);
		this.drawingPaint.setStrokeCap(Paint.Cap.ROUND);
		this.drawingPaint.setStrokeWidth(30);

		this.drawing = new DrawingView(this, this.drawingPaint);

		((LinearLayout) this.findViewById(R.id.drawing_layout))
				.addView(this.drawing);

		((Button) this.findViewById(R.id.photo)).setOnClickListener(this.onPictureClickListener);
		((Button) this.findViewById(R.id.clear)).setOnClickListener(this.onClearClickListener);
		((Button) this.findViewById(R.id.color)).setOnClickListener(this.onColorClickListener);

		((Button) this.findViewById(R.id.lock)).setOnClickListener(this.onLockClickListener);

		this.initGestures();
	}

	@Override
	protected void dispatchCaptureIntent()
	{
		this.clearDrawing();

		super.dispatchCaptureIntent();
	}

	@Override
	public void onBackPressed()
	{
		if (!this.locked) {
			super.onBackPressed();
		}
	}

	protected void initGestures()
	{
		((LinearLayout) this.findViewById(R.id.lock_container)).setOnTouchListener(new View.OnTouchListener()
		{
			protected float down_x, up_x;

			@Override
			public boolean onTouch(View v, MotionEvent event)
			{
				switch (event.getAction()) {
					case MotionEvent.ACTION_DOWN: {
						this.down_x = event.getX();

						return true;
					}
					case MotionEvent.ACTION_UP: {
						this.up_x = event.getX();

						float deltaX = this.down_x - this.up_x;

						if (Math.abs(deltaX) > getResources().getDisplayMetrics().widthPixels * LOCK_SWIPE_OFFSET) {
							if (0 > deltaX || 0 < deltaX) {
								this.onSwipe();

								return true;
							}
						}
					}
				}

				return false;
			}

			private void onSwipe()
			{
				unlock();
			}
		});
	}

	protected void lock()
	{
		this.locked = true;
		this.drawing.lock();

		((Button) this.findViewById(R.id.photo)).setVisibility(View.INVISIBLE);
		((Button) this.findViewById(R.id.clear)).setVisibility(View.INVISIBLE);
		((Button) this.findViewById(R.id.color)).setVisibility(View.INVISIBLE);

		((Button) this.findViewById(R.id.lock)).setVisibility(View.INVISIBLE);

		Toast.makeText(this, "Aby odblokować przeciągnij poziomo u dołu ekranu w lewo lub prawo ...", Toast.LENGTH_LONG).show();
	}

	protected void unlock()
	{
		this.locked = false;
		this.drawing.unlock();

		((Button) this.findViewById(R.id.photo)).setVisibility(View.VISIBLE);
		((Button) this.findViewById(R.id.clear)).setVisibility(View.VISIBLE);
		((Button) this.findViewById(R.id.color)).setVisibility(View.VISIBLE);

		((Button) this.findViewById(R.id.lock)).setVisibility(View.VISIBLE);
	}

	protected void clearDrawing()
	{
		this.drawing.clear();
	}

	protected void setDrawingColor(int color)
	{
		this.drawingColor = color;
		this.drawingPaint.setColor(color);

		this.drawing.setContextPaint(this.drawingPaint);
	}

	protected void openColorDialog()
	{
		ColorPickerDialog dialog = ColorPickerDialog.getInstance(
				COLORS, this.drawingColor, COLORS_COLUMNS
		);
		dialog.setOnColorSelectedListener(this.onColorSelectedListener);
		dialog.show(this.getFragmentManager(), "picker");
	}

	Button.OnClickListener onPictureClickListener =
			new Button.OnClickListener()
			{
				@Override
				public void onClick(View v)
				{
					dispatchCaptureIntent();

					playSound(R.raw.click, false);
				}
			};

	Button.OnClickListener onClearClickListener =
			new Button.OnClickListener()
			{
				@Override
				public void onClick(View v)
				{
					clearDrawing();

					playSound(R.raw.click, false);
				}
			};

	Button.OnClickListener onColorClickListener =
			new Button.OnClickListener()
			{
				@Override
				public void onClick(View v)
				{
					openColorDialog();

					playSound(R.raw.click, false);
				}
			};

	Button.OnClickListener onLockClickListener =
			new Button.OnClickListener()
			{
				@Override
				public void onClick(View v)
				{
					lock();

					playSound(R.raw.click, false);
				}
			};

	ColorPickerDialog.OnColorSelectedListener onColorSelectedListener =
			new ColorPickerDialog.OnColorSelectedListener()
			{
				@Override
				public void onColorSelected(int color)
				{
					setDrawingColor(color);
				}
			};
}
