package com.compo.camera.activities.main;

import android.Manifest;
import android.app.Activity;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.media.ExifInterface;
import android.media.MediaPlayer;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.provider.MediaStore;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.Toast;

import com.compo.camera.R;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

public class MainActivity extends Activity
{
	protected static final int ACTION_TAKE_PHOTO = 2;
	protected static final String STORAGE_KEY = "storage_bitmap";

	protected static final int PERMISSIONS_CODE_CAMERA = 7;
	protected static final int PERMISSIONS_CODE_WRITE_EXTERNAL_STORAGE = 8;

	protected static final float TEXTERE_FIXED_MAX_SIZE = 2048.0f;

	protected ImageView view;
	protected Uri uri;

	protected boolean exit;

	protected MediaPlayer media;

	/**
	 * Called when the activity is first created.
	 */
	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		this.assertPermissions();

		this.requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.getWindow().setFlags(
				WindowManager.LayoutParams.FLAG_FULLSCREEN | WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON,
				WindowManager.LayoutParams.FLAG_FULLSCREEN);
		this.setContentView(R.layout.main);

		this.playSound(R.raw.start_jingle, false);

		this.view = (ImageView) findViewById(R.id.image);
		this.view.setPadding(25, 0, 25, 0);

		this.exit = false;

		this.view.setImageResource(R.drawable.tutorial);
		this.view.setVisibility(View.VISIBLE);
	}

	@Override
	public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults)
	{
		this.assertPermissions();
	}

	protected void playSound(int id, boolean loop)
	{
		try {
			if (null == this.media) {
				this.media = MediaPlayer.create(this, id);
			} else if (this.media.isPlaying()) {
				this.media.stop();
				this.media.release();
			}

			this.media = MediaPlayer.create(this, id);
			this.media.start();
		} catch (Exception e) {
			Log.e(MainActivity.class.toString(), "Media player error for '" + id + "' sound!");
		}
	}

	protected void dispatchCaptureIntent()
	{
		File photo = new File(Environment.getExternalStorageDirectory(), "picture");

		this.uri = Uri.fromFile(photo);

		Intent intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
		intent.putExtra(MediaStore.EXTRA_OUTPUT, this.uri);

		this.startActivityForResult(intent, ACTION_TAKE_PHOTO);
	}

	protected void handlePhoto(Intent intent)
	{
		this.view.setImageURI(null);
		this.view.setPadding(0, 0, 0, 0);

		this.view.setImageBitmap(this.getModifiedBitmap());
		this.view.setVisibility(View.VISIBLE);
	}

	protected Bitmap getModifiedBitmap()
	{
		Bitmap output = null;

		try {
			Bitmap resized = this.getResizedBitmap(this.getBitmapSample());

			output = Bitmap.createBitmap(resized, 0, 0, resized.getWidth(), resized.getHeight(), this.getRotationTransform(), true);
		} catch (IOException e) {
			Toast.makeText(this, "Nie można odnaleźć pliku!", Toast.LENGTH_SHORT).show();
		} catch (OutOfMemoryError e) {
			Toast.makeText(this, "Twoje zdjęcie jest zbyt duże. Zmniejsz rozdzielczość kamery!", Toast.LENGTH_SHORT).show();
		}

		return output;
	}

	private Bitmap getResizedBitmap(int sample) throws FileNotFoundException
	{
		BitmapFactory.Options options = new BitmapFactory.Options();
		options.inSampleSize = sample;

		return BitmapFactory.decodeStream(this.getContentResolver().openInputStream(this.uri), null, options);
	}

	protected Matrix getRotationTransform() throws IOException
	{
		Matrix transform = new Matrix();

		switch (new ExifInterface(new File(this.uri.getPath()).getPath())
				.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_NORMAL)) {
			case ExifInterface.ORIENTATION_ROTATE_90:
				transform.postRotate(90);
				break;
			case ExifInterface.ORIENTATION_ROTATE_180:
				transform.postRotate(180);
				break;
			case ExifInterface.ORIENTATION_ROTATE_270:
				transform.postRotate(270);
				break;
		}

		return transform;
	}

	protected int getBitmapSample()
	{
		BitmapFactory.Options options = new BitmapFactory.Options();
		options.inJustDecodeBounds = true;

		BitmapFactory.decodeFile(new File(this.uri.getPath()).getAbsolutePath(), options);

		int sample = 1;

		int desired_width = options.outWidth;
		int desired_height = options.outHeight;

		while (desired_width > TEXTERE_FIXED_MAX_SIZE && desired_height > TEXTERE_FIXED_MAX_SIZE) {
			desired_width /= 2;
			desired_height /= 2;

			sample *= 2;
		}

		return sample;
	}

	protected void delay()
	{
		new Handler().postDelayed(new Runnable()
		{
			@Override
			public void run()
			{
				exit = false;
			}
		}, 3 * 1000);
	}

	protected void assertPermissions()
	{
		if (!this.checkPermissions(Manifest.permission.CAMERA)) {
			ActivityCompat.requestPermissions(this,
					new String[]{Manifest.permission.CAMERA},
					PERMISSIONS_CODE_CAMERA);
		} else if (!this.checkPermissions(Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
			ActivityCompat.requestPermissions(this,
					new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
					PERMISSIONS_CODE_WRITE_EXTERNAL_STORAGE);
		}
	}

	protected boolean checkPermissions(String permission)
	{
		return PackageManager.PERMISSION_GRANTED == ContextCompat.checkSelfPermission(this, permission);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		MenuInflater inflater = this.getMenuInflater();
		inflater.inflate(R.menu.menu, menu);

		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public void onBackPressed()
	{
		if (this.exit) {
			this.finish();
		} else {
			Toast.makeText(this, "Naciśnij ponownie, aby wyjść ...", Toast.LENGTH_SHORT).show();

			this.exit = true;
			this.delay();
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		switch (requestCode) {
			case ACTION_TAKE_PHOTO: {
				if (RESULT_OK == resultCode) {
					this.handlePhoto(data);
				}
				break;
			}
		}
	}

	@Override
	protected void onSaveInstanceState(Bundle outState)
	{
		outState.putParcelable(STORAGE_KEY, this.uri);

		super.onSaveInstanceState(outState);
	}

	@Override
	protected void onRestoreInstanceState(Bundle savedInstanceState)
	{
		super.onRestoreInstanceState(savedInstanceState);

		this.uri = savedInstanceState.getParcelable(STORAGE_KEY);

		this.view.setImageURI(this.uri);
		this.view.setVisibility(ImageView.VISIBLE);
	}
}