package com.compo.camera.view;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.PorterDuff;
import android.view.MotionEvent;
import android.view.View;

/**
 * Created by lmodlinski on 25/03/17.
 */
public class DrawingView extends View
{
	protected static final float DRAG_OFFSET = 4.0f;
	protected static final int DRAG_RADIUS = 30;

	protected boolean locked;
	protected float drag_x, drag_y;

	protected Bitmap bitmap;
	protected Canvas canvas;

	protected Path path;
	protected Paint paint;

	protected Paint circlePaint;
	protected Path circlePath;

	protected Paint contextPaint;

	public DrawingView(Context context, Paint contextPaint)
	{
		super(context);

		this.contextPaint = contextPaint;

		this.locked = false;

		this.path = new Path();
		this.paint = new Paint(Paint.DITHER_FLAG);

		this.circlePath = new Path();

		this.circlePaint = new Paint();
		this.circlePaint.setAntiAlias(true);
		this.circlePaint.setColor(Color.BLUE);
		this.circlePaint.setStyle(Paint.Style.STROKE);
		this.circlePaint.setStrokeJoin(Paint.Join.MITER);
		this.circlePaint.setStrokeWidth(4f);
	}

	/**
	 * Set paint color_selected by user
	 *
	 * @param paint
	 */
	public void setContextPaint(Paint paint)
	{
		this.contextPaint = paint;
	}

	/**
	 * Lock drawing with finger drag
	 */
	public void lock()
	{
		this.locked = true;
	}

	/**
	 * Unlock drawing with finger drag
	 */
	public void unlock()
	{
		this.locked = false;
	}

	/**
	 * Clear canvas
	 */
	public void clear()
	{
		this.canvas.drawColor(0, PorterDuff.Mode.CLEAR);

		this.invalidate();
	}

	@Override
	protected void onSizeChanged(int w, int h, int oldw, int oldh)
	{
		super.onSizeChanged(w, h, oldw, oldh);

		this.bitmap = Bitmap.createBitmap(w, h, Bitmap.Config.ARGB_8888);
		this.canvas = new Canvas(this.bitmap);
	}

	@Override
	protected void onDraw(Canvas canvas)
	{
		super.onDraw(canvas);

		canvas.drawBitmap(this.bitmap, 0, 0, this.paint);
		canvas.drawPath(this.path, this.contextPaint);
		canvas.drawPath(this.circlePath, this.circlePaint);
	}

	@Override
	public boolean onTouchEvent(MotionEvent event)
	{
		if (!this.locked) {
			float x = event.getX();
			float y = event.getY();

			switch (event.getAction()) {
				case MotionEvent.ACTION_DOWN:
					this.start(x, y);
					this.invalidate();
					break;
				case MotionEvent.ACTION_MOVE:
					this.move(x, y);
					this.invalidate();
					break;
				case MotionEvent.ACTION_UP:
					this.up();
					this.invalidate();
					break;
			}
		}

		return true;
	}

	/**
	 * Whenever screen is touched
	 *
	 * @param x
	 * @param y
	 */
	private void start(float x, float y)
	{
		this.path.reset();
		this.path.moveTo(x, y);

		this.drag_x = x;
		this.drag_y = y;
	}

	/**
	 * Whenever dragging
	 *
	 * @param x
	 * @param y
	 */
	private void move(float x, float y)
	{
		if (Math.abs(x - this.drag_x) >= DRAG_OFFSET
				|| Math.abs(y - this.drag_y) >= DRAG_OFFSET) {
			this.path.quadTo(
					this.drag_x, this.drag_y,
					(int) ((x + this.drag_x) * 0.5),
					(int) ((y + this.drag_y) * 0.5)
			);

			this.drag_x = x;
			this.drag_y = y;

			this.circlePath.reset();
			this.circlePath.addCircle(this.drag_x, this.drag_y, DRAG_RADIUS, Path.Direction.CW);
		}
	}

	/**
	 * Whenever touch is released
	 */
	private void up()
	{
		this.path.lineTo(this.drag_x, this.drag_y);

		this.circlePath.reset();
		this.canvas.drawPath(this.path, this.contextPaint);
		this.path.reset();

	}
}
